--!parent
--!encoding=utf-8

--****************************************************************************
-- @file			LSW CORE LIB: QTWIDGETS (OOP API VERSION)
-- @author   	Valentin Schmidt
-- @version  	0.2
-- @requires  Xtra("QtWidgets")
--****************************************************************************

property _qx
property _confFile

property _dropHandler
property _dropTarget

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
	$ = me.$
	
  if $.os="mac" then die("Error: No Mac Version yet!")
  		
	-- check/load xtra
  $.loadXtras(me.libdir, ["QtWidgets.x32"], [])
		
	-- libs
	$.import("lib/detacher")
	$.import("lib/file")
	$.import("lib/filesystem")
	
	-- CREATE TEMPORARY QT.CONF IN EXE FOLDER
	if $.os="win" then
		tPluginPath = me.libdir & "win\plugins" -- 	"C:\dev\LSW\lib\qt\plugins"
		len = tPluginPath.length
		repeat with i = len down to 1
			if tPluginPath.char[i] = "\" then put "\" after char i of tPluginPath
		end repeat
		-- put tPluginPath -- "C:\\dev\\LSW\\lib\\qt\\plugins"
	else
		-- NO MAC VERSION YET
	end if
	confFile = _player.applicationPath & "qt.conf"
	tString = "[Paths]"&_global.$.CRLF&"Plugins="&tPluginPath -- .\\lib\\qt\\plugins"
	ok = $.file.putString(confFile, tString)
	if ok then me._confFile = confFile
	
	-- LOAD XTRA
	me._qx = xtra("QtWidgets").new()
	
	-- AUTOLOAD ALL WIDGET/DIALOG/OBJECT CLASSES
	--ms = the milliseconds
	me._autoloadClasses()
	--put (the milliseconds-ms)

	-- LOAD EXTENSIONS		
	$.inherit(me, $.include(me.libdir&"constants.ls"))
	$.inherit(me, $.include(me.libdir&"events.ls"))
	
	return me
end

----------------------------------------
-- 
----------------------------------------
on setDropHandler (me, dropHandler, dropTarget)
	me._dropHandler = dropHandler
	me._dropTarget = dropTarget
	me._qx.setDropCallback(#_detachDrop, me)
end

----------------------------------------
-- 
----------------------------------------
on equal (me, obj1, obj2)
	if ilk(obj1)<>#instance OR ilk(obj2)<>#instance then return (obj1=obj2)
	if obj1._class<>obj2._class then return false
	return (obj1._id=obj2._id)
end

----------------------------------------
-- 
----------------------------------------
on _autoloadClasses (me)
	fslib = me.$.filesystem
	
	od = _player.itemdelimiter
	_player.itemdelimiter = "."
	
	types = ["ls","js"]
	
	-- load dialog classes
	filenames = fslib.folderList(me.libdir&"classes/dialog/")
	repeat with f in filenames
		if types.getPos(the last item of f)=0 then next repeat
		me[symbol(f.item[1])] = me.$.include(me.libdir&"classes/dialog/"&f)
	end repeat
	
	-- load object classes
	filenames = fslib.folderList(me.libdir&"classes/object/")
	repeat with f in filenames
		if types.getPos(the last item of f)=0 then next repeat
		me[symbol(f.item[1])] = me.$.include(me.libdir&"classes/object/"&f)
	end repeat

	-- load widget classes
	filenames = fslib.folderList(me.libdir&"classes/widget/")
	repeat with f in filenames
		if types.getPos(the last item of f)=0 then next repeat
		me[symbol(f.item[1])] = me.$.include(me.libdir&"classes/widget/"&f)
	end repeat
	
	_player.itemdelimiter = od
end

----------------------------------------
-- @callback
----------------------------------------
on __destruct (me)
  _qx = 0
	if stringP(me._confFile) then
		fp = xtra("fileIO").new()
		fp.openFile(me._confFile, 0)
		err = fp.status()
		if (err=0) then fp.delete()
		fp = 0
	end if
end

----------------------------------------
-- Used for "detaching" drop action xtra callbacks from original thread,
-- so scripting errors in callbacks don't fail silently
-- @callback
----------------------------------------
on _detachDrop (me, data, p)
	me.$.detacher.detach(me._dropHandler, me._dropTarget, data, p)
end
