--!parent
--!encoding=utf-8

--****************************************************************************
-- @file      TreeWidget Demo - main class
-- @author    Valentin Schmidt
-- @version   0.1
--****************************************************************************

global $

property pAppName

-- widgets
property pMainWindow
property pTreeWidget
property pMenuBar
property pContextMenu

-- other vars
property pRootItem
property pItemDict
--property pIconPath

----------------------------------------
-- @constructor
----------------------------------------
on new(me)

  me.pAppName = "TreeWidgetDemo"

  -- SET DEFAULTS
--  me.pDefaultFont = $.Qt.Font.new()
--  me.pDefaultFont.setFamily("Courier New")
--  me.pDefaultFont.setPointSize(10.0) -- Arial
--  me.pDefaultFont.setBold(FALSE)
--  me.pDefaultFont.setItalic(FALSE)
--  me.pDefaultFont.setUnderline(FALSE)

  --me.pIconPath = $.@("images/"&$.os&"/")

  -- CREATE MAIN WINDOW
  me.pMainWindow = $.Qt.MainWindow.new()
  me.pMainWindow.connect($.Qt.Event.Close, me, #slotQuit)

  me.pMainWindow.setWindowTitle(me.pAppName)
  me.pMainWindow.setWindowIcon($.@("images/logo.png"))
  me.pMainWindow.setToolButtonStyle($.Qt.ToolButtonFollowStyle)

  -- CREATE TREEWIDGET
  me.pTreeWidget = $.Qt.TreeWidget.new(me.pMainWindow)
  me.pMainWindow.setCentralWidget(me.pTreeWidget)
--  me.pTreeWidget.setColumnCount(2)
--  me.pTreeWidget.setHeaderLabels(["Tree", "Description"])
  me.pTreeWidget.setHeaderLabel("TreeWidget Demo")
	me.pTreeWidget.setMouseTracking(TRUE)

	-- custom style
	-- me.pTreeWidget.setStyleSheet("QTreeWidget::item { border-bottom: 1px solid black;}")

	me.pTreeWidget.setDragDropMode($.Qt.QAbstractItemView.InternalMove)

	me.pItemDict = [:]

	-- ADD SOME DUMMY ITEMS
  me.pRootItem = $.Qt.TreeWidgetItem.new(me.pTreeWidget)
  me.makeUID(me.pRootItem)
 	me.pRootItem.setText(0, "Root")
  me.pTreeWidget.addTopLevelItem(me.pRootItem)

  repeat with i = 1 to 5

		childItem = $.Qt.TreeWidgetItem.new(me.pRootItem)
		me.makeUID(childItem)
		childItem.setText(0, "Child_"&i)
		childItem.setFlags(bitOr(childItem.flags(),$.Qt.ItemIsEditable))
		me.pRootItem.addChild(childItem)

    repeat with j = 1 to 5
    	grandChildItem = $.Qt.TreeWidgetItem.new(me.pRootItem)
    	me.makeUID(grandChildItem)
			grandChildItem.setText(0, "Child_"&i&"_"&j)
			grandChildItem.setFlags(bitOr(grandChildItem.flags(),$.Qt.ItemIsEditable))
			childItem.addChild(grandChildItem)
		end repeat

  end repeat

	-- DEACTIVATE DROP FOR INVISIBLE ROOT (=TOP LEVEL)
--	invisibleRootItem = me.pTreeWidget.invisibleRootItem()
--	flags = invisibleRootItem.flags()
--	invisibleRootItem.setFlags(bitAnd(flags, bitNot($.Qt.ItemIsDropEnabled)))

	-- DEACTIVATE DROP FOR VISIBLE SINGLE ROOT ITEM
	--flags = me.pRootItem.flags()
	--me.pRootItem.setFlags(bitAnd(flags, bitNot($.Qt.ItemIsDropEnabled)))

  -- CREATE MENUBAR
  me.pMenuBar = $.Qt.MenuBar.new(me.pMainWindow)
  me.pMainWindow.setMenuBar(me.pMenuBar)

	me.setupFileActions()
	me.setupHelpActions()

	-- CREATE CONTEXT-MENU
	me.setupContextMenu()
	me.pTreeWidget.setContextMenuPolicy($.Qt.CustomContextMenu)
	me.pTreeWidget.connect(#customContextMenuRequested, me, #customContextMenuRequested)

  -- CREATE STATUSBAR
  statusBar = $.Qt.StatusBar.new(pMainWindow)
  me.pMainWindow.setStatusBar(statusBar)

	-- SETUP TREEWIDGET SIGNALS
--	me.pTreeWidget.connect(#currentItemChanged, me, #currentItemChanged)
--	me.pTreeWidget.connect(#itemActivated, me, #itemActivated)
--	me.pTreeWidget.connect(#itemChanged, me, #itemChanged)
--	me.pTreeWidget.connect(#itemClicked, me, #itemClicked)
--	me.pTreeWidget.connect(#itemCollapsed, me, #itemCollapsed)
--	me.pTreeWidget.connect(#itemDoubleClicked, me, #itemDoubleClicked)
--	me.pTreeWidget.connect(#itemEntered, me, #itemEntered)
--	me.pTreeWidget.connect(#itemExpanded, me, #itemExpanded)
--	me.pTreeWidget.connect(#itemPressed, me, #itemPressed)
--	me.pTreeWidget.connect(#itemSelectionChanged, me, #itemSelectionChanged)

  -- RESIZE AND SHOW MAIN WINDOW
	settings = $.Qt.Settings.new("Fluxus", me.pAppName, $.Qt.QSettings.IniFormat)
	state = value(settings.value("mainwindow/state"))
	if state=2 then
		me.pMainWindow.resize(320, 480)
		me.pMainWindow.setWindowState(state)
	else
	  rect = value(settings.value("mainwindow/rect"))
	  if ilk(rect)=#rect then me.pMainWindow.setGeometry(rect)
	  else me.pMainWindow.resize(680, 680)
  end if
  me.pMainWindow.show()

  return me
end

--------------------------------------
-- Helper function, creates unique ID for treeItem, saves it both inside the
-- treeItem (as user data) and in me.pItemDict. This allows to identify items in
-- callback functions
-- @param {instance} treeWidgetItem
--------------------------------------
on makeUID (me, treeWidgetItem)
	uid = string(treeWidgetItem).word[4]
	delete the last char of uid
	me.pItemDict[uid] = treeWidgetItem
	treeWidgetItem.setData(0, $.Qt.UserRole, uid)
end

--------------------------------------
--
--------------------------------------
on setupContextMenu me
  me.pContextMenu = $.Qt.Menu.new(me.pMainWindow)

 -- action "Add Child"
	action = $.Qt.Action.new(me.pContextMenu)
	action.setText("Add Child Item")
	--action.setShortcut("Ctrl+N")
	action.connect(#triggered, me, #addChild)
	me.pContextMenu.addAction(action)

 	-- action "Delete"
	action = $.Qt.Action.new(me.pContextMenu)
	action.setText("Delete Item")
	--action.setShortcut("Ctrl+N")
	action.connect(#triggered, me, #deleteItem)
	me.pContextMenu.addAction(action)
end

--------------------------------------
--
--------------------------------------
on setupFileActions(me)

  --toolBar = $.Qt.ToolBar.new(me.pMainWindow)
  --toolBar.setWindowTitle("File Actions")
  --me.pMainWindow.addToolBar(toolBar)

  menu = me.pMenuBar.addMenu("&File")

	--  action = $.Qt.Action.new(menu)
	--  action.setText("&New")
	--  action.setIcon(me.pIconPath & "filenew.png")
	--  action.connect(#triggered, me, #fileNew)
	--  action.setShortcut("Ctrl+N")

  --  toolBar.addAction(action)
  --  menu.addAction(action)
	--  menu.addSeparator()

  action = $.Qt.Action.new(menu)
  action.setText("&Quit")
	action.connect(#triggered, me, #slotQuit)
  menu.addAction(action)
end

--------------------------------------
--
--------------------------------------
on setupHelpActions(me)

  menuHelp = me.pMenuBar.addMenu("Help")

  action = $.Qt.Action.new(menuHelp)
  action.setText("About &"&me.pAppName)
  action.connect(#triggered, me, #slotAbout)
  menuHelp.addAction(action)

  action = $.Qt.Action.new(menuHelp)
  action.setText("About &Qt")
  action.connect(#triggered, me, #slotAboutQt)
  menuHelp.addAction(action)
 end

--------------------------------------
-- @callback both action and event callback
--------------------------------------
on slotQuit(me)

	--save window settings
	settings = $.Qt.Settings.new("Fluxus", me.pAppName, $.Qt.QSettings.IniFormat)
	settings.setValue("mainwindow/rect", string(me.pMainWindow.geometry()))
  settings.setValue("mainwindow/state", string(me.pMainWindow.windowState()))
	
  halt()
end

--------------------------------------
-- @callback object callback (called by action)
--------------------------------------
on slotAbout (me)
  msg = "Qt TreeWidget Demo written in Lingo<br><br>"&\
  "This application is based on a small runtime engine I called 'LSW', "&\
  "it was written in a code editor (UltraEdit) without ever touching the Director IDE.<br><br>"&\
  "The idea of LSW is to write batch scripts (e.g.for daily automation tasks etc.) or "&\
  "desktop applications in Lingo, in a way similar to e.g. Node.js or Python.<br><br>"&\
  "More infos on LSW <a href='http://valentin.dasdeck.com/lingo/lsw/'>here</a>."

  $.Qt.MessageBox.about(me.pMainWindow, "About "&me.pAppName, msg)
end

--------------------------------------
-- @callback object callback (called by action)
--------------------------------------
on slotAboutQt (me)
  $.Qt.Application.aboutQt()
end

----------------------------------------
-- @callback
----------------------------------------
on currentItemChanged (me, widgetID)
	put "currentItemChanged", me.pTreeWidget.currentItem()
end

----------------------------------------
-- @callback
----------------------------------------
on itemSelectionChanged (me, widgetID)
	put "itemSelectionChanged"
end

----------------------------------------
-- @callback
----------------------------------------
on itemExpanded (me, widgetID, str, data)
	put "itemExpanded", str, data
	--treeWidgetItem = me.pItemDict[data]
end

----------------------------------------
-- @callback
----------------------------------------
on itemCollapsed (me, widgetID, str, data)
	put "itemCollapsed", str, data
	--treeWidgetItem = me.pItemDict[data]
end

----------------------------------------
-- @callback
----------------------------------------
on itemClicked (me, widgetID, str, data)
	put "itemClicked", str, data
	treeWidgetItem = me.pItemDict[data]
	me.pTreeWidget.editItem(treeWidgetItem, 0)
end

----------------------------------------
-- @callback
----------------------------------------
on itemDoubleClicked (me, widgetID, str, data)
	--put "itemDoubleClicked", str, data
	treeWidgetItem = me.pItemDict[data]
	--put treeWidgetItem.childCount()
	
	-- double click moves children of clicked item to root item
	items = treeWidgetItem.takeChildren()
	me.pRootItem.addChildren(items)
end

----------------------------------------
-- @callback
----------------------------------------
on itemChanged (me, widgetID, str, data)
	put "itemChanged", str, data
	--treeWidgetItem = me.pItemDict[data]
end

----------------------------------------
-- @callback
----------------------------------------
on itemActivated (me, widgetID, str, data)
	put "itemActivated", str, data
	--treeWidgetItem = me.pItemDict[data]
end

----------------------------------------
-- @callback
----------------------------------------
on itemEntered (me, widgetID, str, data)
	put "itemEntered", str, data
	--treeWidgetItem = me.pItemDict[data]
end

----------------------------------------
-- @callback
----------------------------------------
on itemPressed (me, widgetID, str, data)
	put "itemPressed", str, data
	--treeWidgetItem = me.pItemDict[data]
end


----------------------------------------
-- @callback
----------------------------------------
on customContextMenuRequested (me, widgetID, p)
	me.pContextMenu.exec()
end

----------------------------------------
-- @callback
----------------------------------------
on addChild (me, widgetID)
	currItem = me.pTreeWidget.currentItem() -- erzeugt jedes mal neue instanz!
	-- tTitle, tLabelText, tEchoMode, tText)
	res = $.Qt.InputDialog.getText(me.pMainWindow, "New Item","Name of new item?", $.Qt.QLineEdit.Normal, "New")
	if string(res)<>"" then
		childItem = $.Qt.TreeWidgetItem.new(me.pRootItem)
		me.makeUID(childItem)
		childItem.setText(0, res)
		childItem.setFlags(bitOr(childItem.flags(),$.Qt.ItemIsEditable))
		currItem.addChild(childItem)
	end if
end

----------------------------------------
-- @callback
----------------------------------------
on deleteItem (me, widgetID)
	currItem = me.pTreeWidget.currentItem()
	btns = bitOr($.Qt.QMessageBox.Ok, $.Qt.QMessageBox.Cancel)
	res = $.Qt.MessageBox.question(me.pMainWindow, "Confirmation" ,"Really delete item?", btns)
	if res=$.Qt.QMessageBox.Ok then
		if $.Qt.equal(currItem, me.pRootItem) then
			invisibleRootItem = me.pTreeWidget.invisibleRootItem()	
			invisibleRootItem.removeChild(currItem)
			me.pRootItem = 0
		else
			parentItem = currItem.parent()
			parentItem.removeChild(currItem)
		end if
		--currItem.destroy() -- ?
	end if
end